#!/usr/bin/env python3
"""
Small qmlformat wrapper that warns the user if the tool is not installed.
"""
import argparse
import shutil
import subprocess
import pathlib
import sys
import re

QMLFORMAT_MISSING_MESSAGE = """
qmlformat is not installed or not in your $PATH, please install.
"""


def find_qt_version():
    moc_executable = shutil.which("moc")
    if not moc_executable:
        return None

    moc_version = subprocess.check_output((moc_executable, "-v")).strip()
    matchobj = re.search("moc ([0-9]*)\\.([0-9]*)\\.[0-9]*", str(moc_version))
    if not matchobj:
        return None

    return (int(matchobj.group(1)), int(matchobj.group(2)))


def main(argv=None):
    qmlformat_executable = None
    # First look up at the most common location for QT6 which is
    # usually not in PATH
    if sys.platform != "win32":
        qmlformat_executable = shutil.which(
            "qmlformat", path="/usr/lib/qt6/bin:/usr/lib64/qt6/bin"
        )
    # Then look in PATH
    if not qmlformat_executable:
        qmlformat_executable = shutil.which("qmlformat")
    if not qmlformat_executable:
        qt_version = find_qt_version()
        if qt_version is None or qt_version < (5, 15):
            # Succeed if a Qt Version < 5.15 is used without qmlformat
            return 0

        print(QMLFORMAT_MISSING_MESSAGE.strip(), file=sys.stderr)
        return 1

    parser = argparse.ArgumentParser()
    parser.add_argument("file", nargs="+", type=pathlib.Path)
    args = parser.parse_args(argv)

    for filename in args.file:
        subprocess.call((qmlformat_executable, "-i", filename))

    return 0


if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import argparse
import logging
import os
import re
import sys
import typing
import fnmatch

from lxml import etree
import githelper

# Define the path to the allow list XML file
ALLOW_LIST_PATH = "res/translations/source_copy_allow_list.tsv"
PROPOSED_ALLOW_LIST_PATH = (
    "res/translations/source_copy_allow_list_proposed.tsv"
)


def parse_line(line, info):
    if not line.endswith("\n"):
        print(f"Parse Error: No \\n at {info}")
        return False
    line = line[:-1]
    try:
        lang, source = line.split("\t", 1)
    except Exception as e:
        print(f"Parse Error at {info}")
        print(e)
    if lang is None or lang == "":
        print(f"Parse Error: lang is empty at {info}")
    return lang, source


def fn_match_comma_sep(language, pattern_cs):
    for lang_pat in pattern_cs.split(","):
        if fnmatch.fnmatchcase(language, lang_pat):
            return True
    return False


def is_untranstaled_allowed(source_text, language):
    source_text = source_text.encode("unicode_escape").decode("utf-8")

    if not os.path.exists(ALLOW_LIST_PATH):
        return False

    with open(ALLOW_LIST_PATH) as f:
        for i, line in enumerate(f):
            lang, source = parse_line(line, f"{ALLOW_LIST_PATH}:{i + 1}")
            if source == source_text:
                return fn_match_comma_sep(language, lang)
    return False


def add_to_allow_list(source_text, language):
    source_text = source_text.encode("unicode_escape").decode("utf-8")

    allow_list_path = PROPOSED_ALLOW_LIST_PATH
    if not os.path.exists(allow_list_path) and os.path.exists(ALLOW_LIST_PATH):
        allow_list_path = ALLOW_LIST_PATH

    tsv_lines = []
    existing_txt = False
    if os.path.exists(allow_list_path):
        with open(allow_list_path) as f:
            tsv_lines = f.readlines()

        # Check if the source_text already exists
        for i, line in enumerate(tsv_lines):
            lang, source = parse_line(line, f"{allow_list_path}:{i + 1}")
            if source == source_text:
                if fn_match_comma_sep(language, lang):
                    return
                lang = lang + "," + language
                tsv_lines[i] = f"{lang}\t{source}\n"
                existing_txt = True
                break
    else:
        tsv_lines.append("lang\tsource\n")

    if not existing_txt:
        tsv_lines.append(f"{language}\t{source_text}\n")

    with open(PROPOSED_ALLOW_LIST_PATH, "w", encoding="utf-8") as tsv_file:
        tsv_file.write("".join(tsv_lines))


def check_copied_source_on_lines(rootdir, file_to_format, stylepath=None):
    filename = os.path.join(rootdir, file_to_format.filename)

    # Extract the language code from the filename
    # (assuming format: mixxx_<lang_code>.ts)
    match = re.search(r".*/mixxx_(\w+)\.ts", file_to_format.filename)
    if not match:
        return False

    lang_code = match.group(1)

    if lang_code.startswith("en_"):
        # for Engish languages all source copies are allowed
        return False

    parser = etree.XMLParser(recover=True)

    try:
        tree = etree.parse(filename, parser)
    except Exception as e:
        print("XML parsing failed:")
        print(e)
        return False

    root = tree.getroot()

    found = False

    for context in root.findall("context"):
        for message in context.findall("message"):
            source = message.find("source").text
            translation_elem = message.find("translation")
            if translation_elem is None:
                continue
            if translation_elem.get("type") == "unfinished":
                continue
            translation = translation_elem.text
            if source != translation:
                continue
            line_number = translation_elem.sourceline
            for start, end in file_to_format.lines:
                if line_number >= start and line_number <= end:
                    if is_untranstaled_allowed(source, lang_code):
                        continue
                    print(
                        "Source copy found at: "
                        f"{file_to_format.filename}:{line_number}\n{source}",
                        file=sys.stderr,
                    )
                    add_to_allow_list(source, lang_code)
                    found = True
    return found


def main(argv: typing.Optional[typing.List[str]] = None) -> int:
    logging.basicConfig(
        format="[%(levelname)s] %(message)s", level=logging.INFO
    )

    parser = argparse.ArgumentParser()
    parser.add_argument("--from-ref", help="use changes changes since commit")
    parser.add_argument("--to-ref", help="use changes until commit")
    parser.add_argument("files", nargs="*", help="only check these files")
    args = parser.parse_args(argv)

    if not args.from_ref:
        args.from_ref = os.getenv("PRE_COMMIT_FROM_REF") or os.getenv(
            "PRE_COMMIT_SOURCE"
        )

    if not args.to_ref:
        args.to_ref = os.getenv("PRE_COMMIT_TO_REF") or os.getenv(
            "PRE_COMMIT_ORIGIN"
        )

    # Filter filenames
    rootdir = githelper.get_toplevel_path()

    files_with_added_lines = githelper.get_changed_lines_grouped(
        from_ref=args.from_ref,
        to_ref=args.to_ref,
        filter_lines=lambda line: line.added,
        include_files=args.files,
    )

    ret = 0
    for changed_file in files_with_added_lines:
        if check_copied_source_on_lines(rootdir, changed_file):
            ret = 1

    if ret:
        print(
            "\n"  # For a distance to the list of findings
            "All disallowed copied source translations need to be removed"
            " at\n"
            "https://app.transifex.com/mixxx-dj-software and pulled again.\n"
            "If they are intended untranslated use:\n"
            f"{PROPOSED_ALLOW_LIST_PATH}\n"
            "to allow it.",
            file=sys.stderr,
        )

    return ret


if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import argparse
import fnmatch
import os
import os.path
import re
import sys
import tinycss.css21


RE_CPP_CLASSNAME = re.compile(r"^\s*class\s+([\w_]+)")
RE_CPP_OBJNAME = re.compile(r'setObjectName\(.*"([^"]+)"')
RE_UI_OBJNAME = re.compile(r'<widget[^>]+name="([^"]+)"')
RE_XML_OBJNAME = re.compile(r"<ObjectName>(.*)</ObjectName>")
RE_XML_OBJNAME_SETVAR = re.compile(
    r'<SetVariable\s+name="ObjectName">(.*)</SetVariable>'
)
RE_CLASSNAME = re.compile(r"^[A-Z]\w+$")
RE_OBJNAME_VARTAG = re.compile(r"<.*>")
# Some object names set via variables in
# src/library/tabledelegates/checkboxdelegate.cpp
CPP_OBJNAMES = [
    "LibraryBPMButton",
    "LibraryPlayedCheckbox",
]

# List of Qt Widgets, generated with:
#     python -c 'import inspect, from PyQt5 import QtWidgets;
#     print([k for k, v in QtWidgets.__dict__.items() if inspect.isclass(v)])'
QTWIDGETS = [
    "QWidget",
    "QAbstractButton",
    "QGraphicsItem",
    "QAbstractGraphicsShapeItem",
    "QAbstractItemDelegate",
    "QFrame",
    "QAbstractScrollArea",
    "QAbstractItemView",
    "QAbstractSlider",
    "QAbstractSpinBox",
    "QAction",
    "QActionGroup",
    "QApplication",
    "QLayoutItem",
    "QLayout",
    "QBoxLayout",
    "QButtonGroup",
    "QCalendarWidget",
    "QCheckBox",
    "QDialog",
    "QColorDialog",
    "QColumnView",
    "QComboBox",
    "QPushButton",
    "QCommandLinkButton",
    "QStyle",
    "QCommonStyle",
    "QCompleter",
    "QDataWidgetMapper",
    "QDateTimeEdit",
    "QDateEdit",
    "QDial",
    "QDialogButtonBox",
    "QDirModel",
    "QDockWidget",
    "QDoubleSpinBox",
    "QErrorMessage",
    "QFileDialog",
    "QFileIconProvider",
    "QFileSystemModel",
    "QFocusFrame",
    "QFontComboBox",
    "QFontDialog",
    "QFormLayout",
    "QGesture",
    "QGestureEvent",
    "QGestureRecognizer",
    "QGraphicsAnchor",
    "QGraphicsLayoutItem",
    "QGraphicsLayout",
    "QGraphicsAnchorLayout",
    "QGraphicsEffect",
    "QGraphicsBlurEffect",
    "QGraphicsColorizeEffect",
    "QGraphicsDropShadowEffect",
    "QGraphicsEllipseItem",
    "QGraphicsGridLayout",
    "QGraphicsItemGroup",
    "QGraphicsLineItem",
    "QGraphicsLinearLayout",
    "QGraphicsObject",
    "QGraphicsOpacityEffect",
    "QGraphicsPathItem",
    "QGraphicsPixmapItem",
    "QGraphicsPolygonItem",
    "QGraphicsWidget",
    "QGraphicsProxyWidget",
    "QGraphicsRectItem",
    "QGraphicsTransform",
    "QGraphicsRotation",
    "QGraphicsScale",
    "QGraphicsScene",
    "QGraphicsSceneEvent",
    "QGraphicsSceneContextMenuEvent",
    "QGraphicsSceneDragDropEvent",
    "QGraphicsSceneHelpEvent",
    "QGraphicsSceneHoverEvent",
    "QGraphicsSceneMouseEvent",
    "QGraphicsSceneMoveEvent",
    "QGraphicsSceneResizeEvent",
    "QGraphicsSceneWheelEvent",
    "QGraphicsSimpleTextItem",
    "QGraphicsTextItem",
    "QGraphicsView",
    "QGridLayout",
    "QGroupBox",
    "QHBoxLayout",
    "QHeaderView",
    "QInputDialog",
    "QItemDelegate",
    "QItemEditorCreatorBase",
    "QItemEditorFactory",
    "QKeyEventTransition",
    "QKeySequenceEdit",
    "QLCDNumber",
    "QLabel",
    "QLineEdit",
    "QListView",
    "QListWidget",
    "QListWidgetItem",
    "QMainWindow",
    "QMdiArea",
    "QMdiSubWindow",
    "QMenu",
    "QMenuBar",
    "QMessageBox",
    "QMouseEventTransition",
    "QOpenGLWidget",
    "QPanGesture",
    "QPinchGesture",
    "QPlainTextDocumentLayout",
    "QPlainTextEdit",
    "QProgressBar",
    "QProgressDialog",
    "QProxyStyle",
    "QRadioButton",
    "QRubberBand",
    "QScrollArea",
    "QScrollBar",
    "QScroller",
    "QScrollerProperties",
    "QShortcut",
    "QSizeGrip",
    "QSizePolicy",
    "QSlider",
    "QSpacerItem",
    "QSpinBox",
    "QSplashScreen",
    "QSplitter",
    "QSplitterHandle",
    "QStackedLayout",
    "QStackedWidget",
    "QStatusBar",
    "QStyleFactory",
    "QStyleHintReturn",
    "QStyleHintReturnMask",
    "QStyleHintReturnVariant",
    "QStyleOption",
    "QStyleOptionButton",
    "QStyleOptionComplex",
    "QStyleOptionComboBox",
    "QStyleOptionDockWidget",
    "QStyleOptionFocusRect",
    "QStyleOptionFrame",
    "QStyleOptionGraphicsItem",
    "QStyleOptionGroupBox",
    "QStyleOptionHeader",
    "QStyleOptionMenuItem",
    "QStyleOptionProgressBar",
    "QStyleOptionRubberBand",
    "QStyleOptionSizeGrip",
    "QStyleOptionSlider",
    "QStyleOptionSpinBox",
    "QStyleOptionTab",
    "QStyleOptionTabBarBase",
    "QStyleOptionTabWidgetFrame",
    "QStyleOptionTitleBar",
    "QStyleOptionToolBar",
    "QStyleOptionToolBox",
    "QStyleOptionToolButton",
    "QStyleOptionViewItem",
    "QStylePainter",
    "QStyledItemDelegate",
    "QSwipeGesture",
    "QSystemTrayIcon",
    "QTabBar",
    "QTabWidget",
    "QTableView",
    "QTableWidget",
    "QTableWidgetItem",
    "QTableWidgetSelectionRange",
    "QTapAndHoldGesture",
    "QTapGesture",
    "QTextEdit",
    "QTextBrowser",
    "QTimeEdit",
    "QToolBar",
    "QToolBox",
    "QToolButton",
    "QToolTip",
    "QTreeView",
    "QTreeWidget",
    "QTreeWidgetItem",
    "QTreeWidgetItemIterator",
    "QUndoCommand",
    "QUndoGroup",
    "QUndoStack",
    "QUndoView",
    "QVBoxLayout",
    "QWhatsThis",
    "QWidgetAction",
    "QWidgetItem",
    "QWizard",
    "QWizardPage",
]


def get_skins(path):
    """Yields (skin_name, skin_path) tuples for each skin directory in path."""
    for entry in os.scandir(path):
        if entry.is_dir():
            yield entry.name, os.path.join(path, entry.name)


def get_global_names(mixxx_path):
    """Returns 2 sets with all class and object names in the Mixxx codebase."""
    classnames = set()
    objectnames = set(CPP_OBJNAMES)
    for root, dirs, fnames in os.walk(os.path.join(mixxx_path, "src")):
        for fname in fnames:
            ext = os.path.splitext(fname)[1]
            if ext in (".h", ".cpp"):
                fpath = os.path.join(root, fname)
                with open(fpath, mode="r") as f:
                    for line in f:
                        classnames.update(set(RE_CPP_CLASSNAME.findall(line)))
                        objectnames.update(set(RE_CPP_OBJNAME.findall(line)))
            elif ext == ".ui":
                fpath = os.path.join(root, fname)
                with open(fpath, mode="r") as f:
                    objectnames.update(set(RE_UI_OBJNAME.findall(f.read())))
    return classnames, objectnames


def get_skin_objectnames(skin_path):
    """
    Yields all object names in the skin_path.

    Note the names may contain one or more <Variable name="x"> tags, so it's
    not enough to check if a name CSS object name is in this list using "in".
    """
    for root, dirs, fnames in os.walk(skin_path):
        for fname in fnames:
            if os.path.splitext(fname)[1] != ".xml":
                continue

            fpath = os.path.join(root, fname)
            with open(fpath, mode="r") as f:
                for line in f:
                    yield from RE_XML_OBJNAME.findall(line)
                    yield from RE_XML_OBJNAME_SETVAR.findall(line)


def get_stylesheets(path):
    """Yields (qss_path, stylesheet) tuples for each qss file in path)."""
    cssparser = tinycss.css21.CSS21Parser()
    for filename in os.listdir(path):
        if os.path.splitext(filename)[1] != ".qss":
            continue
        qss_path = os.path.join(path, filename)
        stylesheet = cssparser.parse_stylesheet_file(qss_path)
        yield qss_path, stylesheet


def check_stylesheet(stylesheet, classnames, objectnames, objectnames_fuzzy):
    """Yields (token, problem) tuples for each problem found in stylesheet."""
    for rule in stylesheet.rules:
        if not isinstance(rule, tinycss.css21.RuleSet):
            continue
        for token in rule.selector:
            if token.type == "IDENT":
                if not RE_CLASSNAME.match(token.value):
                    continue
                if token.value in classnames:
                    continue
                if token.value in QTWIDGETS:
                    continue
                yield (token, 'Unknown widget class "%s"' % token.value)

            elif token.type == "HASH":
                value = token.value[1:]
                if value in objectnames:
                    continue

                if any(
                    fnmatch.fnmatchcase(value, objname)
                    for objname in objectnames_fuzzy
                ):
                    continue

                yield (token, 'Unknown object name "%s"' % token.value)


def check_skins(mixxx_path, skins, ignore_patterns=()):
    """
    Yields error messages for skins using class/object names from mixxx_path.

    By providing a list of ignore_patterns, you can ignore certain class or
    object names (e.g. #Test, #*Debug).
    """
    classnames, objectnames = get_global_names(mixxx_path)

    # Check default stylesheets
    default_styles_path = os.path.join(mixxx_path, "res", "skins")
    for qss_path, stylesheet in get_stylesheets(default_styles_path):
        for error in stylesheet.errors:
            yield "%s:%d:%d: %s - %s" % (
                qss_path,
                error.line,
                error.column,
                error.__class__.__name__,
                error.reason,
            )
        for token, message in check_stylesheet(
            stylesheet, classnames, objectnames, []
        ):
            if any(
                fnmatch.fnmatchcase(token.value, pattern)
                for pattern in ignore_patterns
            ):
                continue
            yield "%s:%d:%d: %s" % (
                qss_path,
                token.line,
                token.column,
                message,
            )

    # Check skin stylesheets
    for skin_name, skin_path in sorted(skins):
        # If the skin objectname is something like 'Deck<Variable name="i">',
        # then replace it with 'Deck*' and use glob-like matching
        skin_objectnames = objectnames.copy()
        skin_objectnames_fuzzy = set()
        for objname in get_skin_objectnames(skin_path):
            new_objname = RE_OBJNAME_VARTAG.sub("*", objname)
            if "*" in new_objname:
                skin_objectnames_fuzzy.add(new_objname)
            else:
                skin_objectnames.add(new_objname)

        for qss_path, stylesheet in get_stylesheets(skin_path):
            for error in stylesheet.errors:
                yield "%s:%d:%d: %s - %s" % (
                    qss_path,
                    error.line,
                    error.column,
                    error.__class__.__name__,
                    error.reason,
                )
            for token, message in check_stylesheet(
                stylesheet,
                classnames,
                skin_objectnames,
                skin_objectnames_fuzzy,
            ):
                if any(
                    fnmatch.fnmatchcase(token.value, pattern)
                    for pattern in ignore_patterns
                ):
                    continue
                yield "%s:%d:%d: %s" % (
                    qss_path,
                    token.line,
                    token.column,
                    message,
                )


def main(argv=None):
    """Main method for handling command line arguments."""
    parser = argparse.ArgumentParser(
        "qsscheck",
        description="Check QSS styles for non-existing object/class names",
    )
    parser.add_argument(
        "-p",
        "--extra-skins-path",
        help="Additional skin path, to check (.e.g. ~/.mixxx/skins)",
    )
    parser.add_argument("-s", "--skin", help="Only check skin with this name")
    parser.add_argument(
        "-i",
        "--ignore",
        default="",
        help="Glob pattern of class/object names to ignore (e.g. '#Test*'), "
        "separated by commas",
    )
    parser.add_argument("mixxx_path", help="Path of Mixxx sources/git repo")
    args = parser.parse_args(argv)

    mixxx_path = args.mixxx_path

    skins_path = os.path.join(mixxx_path, "res", "skins")
    skins = set(get_skins(skins_path))
    if args.extra_skins_path:
        skins.update(set(get_skins(args.extra_skins_path)))

    if args.skin:
        skins = set((name, path) for name, path in skins if name == args.skin)

    if not skins:
        print("No skins to check")
        return 1

    status = 0
    for message in check_skins(mixxx_path, skins, args.ignore.split(",")):
        print(message)
        status = 2
    return status


if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python3
import argparse
import pathlib
import re


def add_missing_links(changelog):
    """Ensure that all the changelog conforms to our common format."""
    # Ensure that all launchpad issues are hyperlinks
    changelog = re.sub(
        r"(?!\[)\blp:?(?P<lpid>\d+)\b(?!\])",
        r"[lp:\g<lpid>](https://bugs.launchpad.net/mixxx/+bug/\g<lpid>)",
        changelog,
    )

    # Ensure that all GitHub PRs are hyperlinks
    changelog = re.sub(
        r"(?!\[)#\b(?P<ghid>\d+)\b(?!\])",
        r"[#\g<ghid>](https://github.com/mixxxdj/mixxx/pull/\g<ghid>)",
        changelog,
    )

    return changelog


def main(argv=None):
    parser = argparse.ArgumentParser()
    parser.add_argument("file", type=pathlib.Path)
    args = parser.parse_args(argv)

    # Fetch changelog and convert to RST
    with args.file.open("r") as fp:
        changelog = fp.read()

    fixed_changelog = add_missing_links(changelog)
    if changelog != fixed_changelog:
        with args.file.open("w") as fp:
            fp.write(fixed_changelog)


if __name__ == "__main__":
    main()



#!/usr/bin/env python
import matplotlib.pyplot as plt
import numpy as np
import argparse
import sys


def createSlice(columns):
    """convert an input string into a slice object"""
    if columns == "all":
        return slice(None)
    # creates a numpy array
    return np.int64(columns.split(","))


def combine_files(files):
    """
    reads a bunch of files and stacks their content together into one numpy
    array. The number of data points in the columns will be set to the
    shortest file.
    """
    raw = []
    min_len = sys.maxsize  # max integer
    for fname in files:
        raw.append(np.genfromtxt(fname, delimiter=","))
        min_len = min(len(raw[-1]), min_len)
    data = raw[0][:min_len]

    for d in raw[1:]:
        data = np.hstack((data, d[:min_len]))
    # in case only one column is given
    if data.ndim == 1:
        data = data.reshape(len(data), 1)
    return data


def AudioPlot(files, slice):
    data = combine_files(files)
    plt.plot(data[:, slice])
    plt.show()


def parseArguments():
    p = argparse.ArgumentParser(
        prog="AudioPlot",
        formatter_class=argparse.RawDescriptionHelpFormatter,
        description="""
Audioplot is a simple script for drawing graphs of values. It reads a simple
csv text file where each line needs to contain the same number of
comma-separated values, like so:

0, 0, 0
1, 2, 1
2, 4, 2

Each column will be plotted as a separate curve on the same time series.
Matplotlib is used to display the result.

This script is useful to compare the sample files produced by the engine test
of mixxx-test with the golden sample files.

""",
    )
    p.add_argument("files", type=str, nargs="+", help="file to plot from")
    p.add_argument(
        "-c",
        "--columns",
        type=str,
        default="all",
        help='lines to plot separated by a comma, default "all"',
    )
    return p.parse_args()


if __name__ == "__main__":
    args = parseArguments()
    AudioPlot(args.files, createSlice(args.columns))



#!/usr/bin/env python3

import argparse
import datetime
import functools
import hashlib
import json
import logging
import os
import pathlib
import shutil
import subprocess
import sys
import urllib.parse
import urllib.request


def url_fetch(url, headers=None, **kwargs):
    """Make a web request to the given URL and return the response object."""
    request_headers = {
        # Override the User-Agent because our download server seems to block
        # requests with the default UA value and responds "403 Forbidden".
        "User-Agent": (
            "Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:86.0) Gecko/20100101 "
            "Firefox/86.0"
        ),
    }
    if headers:
        request_headers.update(headers)
    req = urllib.request.Request(url, headers=request_headers, **kwargs)
    return urllib.request.urlopen(req, timeout=10)


def url_exists(url):
    """Make a HEAD request to the URL and check if the response is "200 OK"."""
    try:
        resp = url_fetch(url, method="HEAD")
    except IOError:
        return False
    return resp.status == 200


def url_download_json(url):
    """Returns the JSON object from the given URL or returns None."""
    resp = url_fetch(url)
    if resp.status != 200:
        raise IOError(f"Server responded with HTTP status {resp.status}")
    manifest_data = resp.read().decode().strip()
    return json.loads(manifest_data)


def sha256(file_path):
    """Returns the sha256 hexdigest for a file."""
    with open(file_path, mode="rb") as fp:
        read_chunk = functools.partial(fp.read, 1024)
        m = hashlib.sha256()
        for data in iter(read_chunk, b""):
            m.update(data)
        return m.hexdigest()


def git_info(info, path="."):
    """Return the checked out git branch for the given path."""
    if info == "branch":
        cmd = ("git", "rev-parse", "--abbrev-ref", "HEAD")
    elif info == "commit":
        cmd = ("git", "rev-parse", "HEAD")
    elif info == "describe":
        # A dirty git state should only be possible on local builds, but since
        # this script may be used locally we'll add it here.
        cmd = (
            "git",
            "describe",
            "--always",
            "--first-parent",
            "--dirty=-modified",
        )
    else:
        raise ValueError("Invalid git info type!")

    return subprocess.check_output(
        cmd,
        cwd=path,
        encoding="utf-8",
    ).strip()


def splitext(filename):
    """
    Split filename into name without extension and file extension.

    This includes a workaround for ".tar.gz" files.
    """
    filename_without_ext, file_ext = os.path.splitext(filename)
    filename_without_ext2, file_ext2 = os.path.splitext(filename_without_ext)
    if file_ext2 == ".tar":
        filename_without_ext = filename_without_ext2
        file_ext = f"{file_ext2}{file_ext}"
    return filename_without_ext, file_ext


def tree(path):
    for dirpath, dirnames, filenames in os.walk(top=path):
        relpath = os.path.relpath(dirpath, start=path)
        if relpath != ".":
            yield relpath
        for filename in filenames:
            yield os.path.join(relpath, filename)


def slug(text):
    download_slug, _, package_slug = text.partition("-")
    if not download_slug or not package_slug:
        raise ValueError("Failed to parse slug")
    return download_slug, package_slug


def prepare_deployment(args):
    logger = logging.getLogger(__name__)

    # Get artifact and build metadata
    try:
        file_stat = os.stat(args.file)
        file_sha256 = sha256(args.file)
    except Exception as e:
        logger.error("Error accessing file: %s", e)
        return 1

    try:
        commit_id = os.environ["GITHUB_SHA"]
    except KeyError:
        commit_id = git_info("commit")

    metadata = {
        "git_commit": commit_id,
        "git_branch": git_info("branch"),
        "git_describe": git_info("describe"),
        "file_size": file_stat.st_size,
        "file_date": datetime.datetime.fromtimestamp(
            file_stat.st_ctime
        ).isoformat(),
        "file_sha256": file_sha256,
    }

    if os.getenv("CI") == "true":
        github_run_id = os.getenv("GITHUB_RUN_ID")
        github_server_url = os.getenv("GITHUB_SERVER_URL")
        github_repository = os.getenv("GITHUB_REPOSITORY")
        metadata.update(
            {
                "git_commit_url": (
                    f"{github_server_url}/{github_repository}/"
                    f"commit/{commit_id}"
                ),
                "build_log_url": (
                    f"{github_server_url}/{github_repository}/actions/"
                    f"runs/{github_run_id}"
                ),
            }
        )

    filename_without_ext, file_ext = splitext(args.file)
    download_slug, package_slug = args.slug

    # Build destination path scheme
    logger.info("Destination path pattern: %s", args.dest_path)
    destpath = args.dest_path.format(
        filename=os.path.basename(args.file),
        filename_without_ext=filename_without_ext,
        ext=file_ext,
        git_branch=metadata["git_branch"],
        git_commit_id=metadata["git_commit"],
        git_describe=metadata["git_describe"],
        package_slug=package_slug,
        download_slug=download_slug,
    )
    logger.info("Destination path: %s", destpath)

    # Move files to deploy in place and create sha256sum file
    output_destpath = os.path.join(args.output_dir, destpath)
    os.makedirs(os.path.dirname(output_destpath), exist_ok=True)
    shutil.copy2(args.file, output_destpath)

    output_filename = os.path.basename(destpath)
    with open(f"{output_destpath}.sha256sum", mode="w") as fp:
        fp.write(f"{file_sha256}  {output_filename}\n")

    metadata.update(
        {
            "file_url": f"{args.dest_url}/{destpath}",
            "file_sha256_url": f"{args.dest_url}/{destpath}.sha256sum",
        }
    )

    # Show metadata and files to deploy
    logger.info("Metadata: %s", json.dumps(metadata, indent=2, sort_keys=True))
    logger.info("Files: \n%s", "\n".join(tree(args.output_dir)))

    # Write metadata to GitHub Actions step output, so that it can be used for
    # manifest creation in the final job after all builds finished.
    if os.getenv("CI") == "true":
        # Set GitHub Actions job output using environment files
        output_file = os.getenv("GITHUB_OUTPUT")
        with open(output_file, "a") as f:
            f.write(
                "artifact-{}-{}={}\n".format(
                    download_slug,
                    package_slug,
                    json.dumps(metadata),
                )
            )
    return 0


def collect_manifest_data(job_data):
    """Parse the job metadata dict and return the manifest data."""
    logger = logging.getLogger(__name__)
    job_result = job_data["result"]
    logger.info("Build job result: %s", job_result)

    manifest_data = {}
    for output_name, output_data in job_data["outputs"].items():
        # Filter out unrelated job outputs that don't start with "artifact-".
        prefix, _, artifact_slug = output_name.partition("-")
        if prefix != "artifact" or not artifact_slug:
            logger.info("Ignoring output '%s'...", output_name)
            continue
        artifact_data = json.loads(output_data)

        url = artifact_data["file_url"]

        # Make sure that the file actually exists on the download server
        try:
            resp = url_fetch(url, method="HEAD")
            if resp.status != 200:
                raise LookupError(
                    f"Unable to find URL '{url}' on remote server"
                )
        except Exception as e:
            logger.error("Error checking URL: %s", e)
            raise

        manifest_data[artifact_slug] = artifact_data

    return manifest_data


def generate_manifest(args):
    logger = logging.getLogger(__name__)
    try:
        commit_id = os.getenv("GITHUB_SHA")
    except KeyError:
        commit_id = git_info("commit")

    format_data = {
        "git_branch": git_info("branch"),
        "git_commit_id": commit_id,
        "git_describe": git_info("describe"),
    }

    # Build destination path scheme
    logger.info("Destination path pattern: %s", args.dest_path)
    destpath = args.dest_path.format_map(format_data)
    logger.info("Destination path: %s", destpath)

    # Create the deployment directory
    output_destpath = os.path.join(args.output_dir, destpath)
    os.makedirs(os.path.dirname(output_destpath), exist_ok=True)

    # Parse the JOB_DATA JSON data, generate the manifest data and print it
    job_data = json.loads(os.environ["JOB_DATA"])
    manifest_data = collect_manifest_data(job_data)

    # If possible, check if the remote manifest is the same as our local one
    remote_manifest_data = {}
    if args.dest_url:
        # Check if generated manifest.json file differs from the one that
        # is currently deployed.
        manifest_url = f"{args.dest_url}/{destpath}"
        manifest_url = manifest_url.format_map(format_data)

        try:
            remote_manifest_data = url_download_json(manifest_url) or {}
        except IOError as e:
            logger.error("Fetching remote manifest failed: %s", e)

    if args.update:
        for key, value in remote_manifest_data.items():
            if key not in manifest_data:
                manifest_data[key] = value

    logger.info(
        "Manifest: %s", json.dumps(manifest_data, indent=2, sort_keys=True)
    )

    # Write the manifest.json for subsequent deployment to the server
    with open(output_destpath, mode="w") as fp:
        json.dump(manifest_data, fp, indent=2, sort_keys=True)

    # Skip deployment if the remote manifest is the same as the local one.
    if manifest_data != remote_manifest_data:
        logger.info("Remote manifest differs from local version.")
        if os.getenv("CI") == "true":
            with open(os.environ["GITHUB_ENV"], mode="a") as fp:
                fp.write("MANIFEST_DIRTY=1\n")
    else:
        logger.info(
            "Remote manifest is the same as local version, skipping upload."
        )

    logger.info("Files:\n%s", "\n".join(tree(args.output_dir)))

    return 0


def main(argv=None):
    parser = argparse.ArgumentParser()
    subparsers = parser.add_subparsers()

    parent_parser = argparse.ArgumentParser(add_help=False)
    parent_parser.add_argument(
        "-v",
        "--verbose",
        action="store_const",
        dest="loglevel",
        const=logging.DEBUG,
        default=logging.INFO,
        help="Increase output verbosity",
    )

    artifact_parser = subparsers.add_parser(
        "prepare-deployment",
        help="Prepare artifact metadata from file",
        parents=[parent_parser],
        add_help=False,
    )
    artifact_parser.set_defaults(cmd=prepare_deployment)
    artifact_parser.add_argument(
        "--slug",
        action="store",
        required=True,
        type=slug,
        help="Artifact identifier for the website's download page",
    )
    artifact_parser.add_argument(
        "--output-dir",
        action="store",
        default="deploy",
        help="Directory to write output to (default: 'deploy')",
    )
    artifact_parser.add_argument(
        "--dest-path",
        action="store",
        required=True,
        help="Destination path inside the output directory",
    )
    artifact_parser.add_argument(
        "--dest-url",
        action="store",
        required=True,
        help="Destination URL prefix",
    )
    artifact_parser.add_argument(
        "file", type=pathlib.Path, help="Local file to deploy"
    )

    manifest_parser = subparsers.add_parser(
        "generate-manifest",
        help="Collect artifact metadata and generate manifest.json file",
        parents=[parent_parser],
        add_help=False,
    )
    manifest_parser.set_defaults(cmd=generate_manifest)
    manifest_parser.add_argument(
        "--output-dir",
        action="store",
        default="deploy",
        help="Directory to write output to (default: 'deploy')",
    )
    manifest_parser.add_argument(
        "--dest-path",
        action="store",
        required=True,
        help="Destination path inside the output directory",
    )
    manifest_parser.add_argument(
        "--dest-url", action="store", help="Destination URL prefix"
    )
    manifest_parser.add_argument(
        "--update",
        action="store_true",
        help="Fetch the remote manifest and update it",
    )

    args = parser.parse_args(argv)

    logging.basicConfig(level=args.loglevel, format="%(message)s")

    if os.path.exists(args.output_dir):
        if not os.path.isdir(args.output_dir):
            raise OSError("Output dir is not a directory!")
        if os.listdir(args.output_dir):
            raise OSError("Output dir is not empty!")

    return args.cmd(args)


if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python
import argparse
import sys

# To use, run this from the top level of the Git repository tree:
# scripts/generate_sample_functions.py
#     --sample_autogen_h src/util/sample_autogen.h

BASIC_INDENT = 4

COPY_WITH_GAIN_METHOD_PATTERN = "copy%(i)dWithGain"


def copy_with_gain_method_name(i):
    return COPY_WITH_GAIN_METHOD_PATTERN % {"i": i}


RAMPING_GAIN_METHOD_PATTERN = "copy%(i)dWithRampingGain"


def copy_with_ramping_gain_method_name(i):
    return RAMPING_GAIN_METHOD_PATTERN % {"i": i}


def method_call(method_name, args):
    return "%(method_name)s(%(args)s)" % {
        "method_name": method_name,
        "args": ", ".join(args),
    }


def hanging_indent(base, groups, hanging_suffix, terminator, depth=0):
    return map(
        lambda line: " " * BASIC_INDENT * depth + line,
        map(
            "".join,
            zip(
                [base] + [" " * len(base)] * (len(groups) - 1),
                groups,
                [hanging_suffix] * (len(groups) - 1) + [terminator],
            ),
        ),
    )


def write_sample_autogen(output, num_channels):
    output.append("#ifndef MIXXX_UTIL_SAMPLEAUTOGEN_H")
    output.append("#define MIXXX_UTIL_SAMPLEAUTOGEN_H")
    output.append("////////////////////////////////////////////////////////")
    output.append("// THIS FILE IS AUTO-GENERATED. DO NOT EDIT DIRECTLY! //")
    output.append("// SEE scripts/generate_sample_functions.py           //")
    output.append("////////////////////////////////////////////////////////")

    for i in range(1, num_channels + 1):
        copy_with_gain(output, 0, i)
        copy_with_ramping_gain(output, 0, i)

    output.append("#endif /* MIXXX_UTIL_SAMPLEAUTOGEN_H */")


def copy_with_gain(output, base_indent_depth, num_channels):
    def write(data, depth=0):
        output.append(
            " " * (BASIC_INDENT * (depth + base_indent_depth)) + data
        )

    header = "static inline void %s(" % copy_with_gain_method_name(
        num_channels
    )
    arg_groups = (
        ["CSAMPLE* M_RESTRICT pDest"]
        + [
            "const CSAMPLE* M_RESTRICT pSrc%(i)d, CSAMPLE_GAIN gain%(i)d"
            % {"i": i}
            for i in range(num_channels)
        ]
        + ["int iNumSamples"]
    )

    output.extend(
        hanging_indent(header, arg_groups, ",", ") {", depth=base_indent_depth)
    )

    for i in range(num_channels):
        write("if (gain%(i)d == CSAMPLE_GAIN_ZERO) {" % {"i": i}, depth=1)
        if num_channels > 1:
            args = (
                ["pDest"]
                + [
                    "pSrc%(i)d, gain%(i)d" % {"i": j}
                    for j in range(num_channels)
                    if i != j
                ]
                + ["iNumSamples"]
            )
            write(
                "%s;"
                % method_call(
                    copy_with_gain_method_name(num_channels - 1), args
                ),
                depth=2,
            )
        else:
            write("clear(pDest, iNumSamples);", depth=2)
        write("return;", depth=2)
        write("}", depth=1)

    write("// note: LOOP VECTORIZED.", depth=1)
    write("for (int i = 0; i < iNumSamples; ++i) {", depth=1)
    terms = [
        "pSrc%(i)d[i] * gain%(i)d" % {"i": i} for i in range(num_channels)
    ]
    assign = "pDest[i] = "
    output.extend(hanging_indent(assign, terms, " +", ";", depth=2))

    write("}", depth=1)
    write("}")


def copy_with_ramping_gain(output, base_indent_depth, num_channels):
    def write(data, depth=0):
        output.append(
            " " * (BASIC_INDENT * (depth + base_indent_depth)) + data
        )

    header = "static inline void %s(" % copy_with_ramping_gain_method_name(
        num_channels
    )
    arg_groups = (
        ["CSAMPLE* M_RESTRICT pDest"]
        + [
            (
                "const CSAMPLE* M_RESTRICT pSrc%(i)d, "
                "CSAMPLE_GAIN gain%(i)din, CSAMPLE_GAIN gain%(i)dout"
            )
            % {"i": i}
            for i in range(num_channels)
        ]
        + ["int iNumSamples"]
    )

    output.extend(
        hanging_indent(header, arg_groups, ",", ") {", depth=base_indent_depth)
    )

    for i in range(num_channels):
        write(
            (
                "if (gain%(i)din == CSAMPLE_GAIN_ZERO && "
                "gain%(i)dout == CSAMPLE_GAIN_ZERO) {"
            )
            % {"i": i},
            depth=1,
        )
        if num_channels > 1:
            args = (
                ["pDest"]
                + [
                    "pSrc%(i)d, gain%(i)din, gain%(i)dout" % {"i": j}
                    for j in range(num_channels)
                    if i != j
                ]
                + ["iNumSamples"]
            )
            write(
                "%s;"
                % method_call(
                    copy_with_ramping_gain_method_name(num_channels - 1), args
                ),
                depth=2,
            )
        else:
            write("clear(pDest, iNumSamples);", depth=2)
        write("return;", depth=2)
        write("}", depth=1)

    for i in range(num_channels):
        write(
            (
                "const CSAMPLE_GAIN gain_delta%(i)d = "
                "(gain%(i)dout - gain%(i)din) / (iNumSamples / 2);"
            )
            % {"i": i},
            depth=1,
        )
        write(
            (
                "const CSAMPLE_GAIN start_gain%(i)d = "
                "gain%(i)din + gain_delta%(i)d;"
            )
            % {"i": i},
            depth=1,
        )

    write("// note: LOOP VECTORIZED.", depth=1)
    write("for (int i = 0; i < iNumSamples / 2; ++i) {", depth=1)

    for i in range(num_channels):
        write(
            (
                "const CSAMPLE_GAIN gain%(i)d = "
                "start_gain%(i)d + gain_delta%(i)d * i;"
            )
            % {"i": i},
            depth=2,
        )

    terms1 = []
    terms2 = []
    for i in range(num_channels):
        terms1.append("pSrc%(i)d[i * 2] * gain%(i)d" % {"i": i})
        terms2.append("pSrc%(i)d[i * 2 + 1] * gain%(i)d" % {"i": i})

    assign1 = "pDest[i * 2] = "
    assign2 = "pDest[i * 2 + 1] = "

    output.extend(hanging_indent(assign1, terms1, " +", ";", depth=2))
    output.extend(hanging_indent(assign2, terms2, " +", ";", depth=2))

    write("}", depth=1)
    write("}")


def main(args):
    sampleutil_output_lines = []
    write_sample_autogen(sampleutil_output_lines, args.max_channels)

    output = (
        open(args.sample_autogen_h, "w")
        if args.sample_autogen_h
        else sys.stdout
    )
    output.write("\n".join(sampleutil_output_lines) + "\n")


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description="Auto-generate sample processing and mixing functions.",
        epilog=(
            "Example Call:"
            "./generate_sample_functions.py --sample_autogen_h "
        ),
    )
    parser.add_argument("--sample_autogen_h")
    parser.add_argument("--max_channels", type=int, default=32)
    args = parser.parse_args()
    main(args)



import os
import itertools
import re
import logging
import subprocess
import typing


Line = typing.NamedTuple(
    "Line",
    [("sourcefile", str), ("number", int), ("text", str), ("added", bool)],
)
FileLines = typing.NamedTuple(
    "FileLines",
    [("filename", str), ("lines", typing.Sequence[typing.Tuple[int, int]])],
)


def get_toplevel_path() -> str:
    logger = logging.getLogger(__name__)

    cmd = ["git", "rev-parse", "--show-toplevel"]
    logger.debug("Executing: %r", cmd)
    return os.path.normpath(subprocess.check_output(cmd, text=True).strip())


def get_moved_files(
    changeset, include_files=None
) -> typing.Iterable[typing.Tuple[str, str]]:
    """
    Inspect `git diff` output to find moved/renamed files.

    Yields tuples in the form (old_file, new_file).

    If include_files is set, this only yields results where at least one of the
    two file names is in the list of file names to include.
    """

    logger = logging.getLogger(__name__)

    cmd = ["git", "diff", "--raw", "-z", changeset]
    logger.debug("Executing: %r", cmd)
    proc = subprocess.run(cmd, capture_output=True)
    proc.check_returncode()
    diff_output = proc.stdout.decode(errors="replace")
    for line in diff_output.lstrip(":").split(":"):
        change, _, files = line.partition("\0")
        _, _, changetype = change.rpartition(" ")
        if changetype.startswith("R"):
            # A file was renamed
            old_file, sep, new_file = files.rstrip("\0").partition("\0")
            assert sep == "\0"

            move = (old_file, new_file)
            if include_files and not any(x in include_files for x in move):
                continue

            yield move


def get_changed_lines(
    from_ref=None, to_ref=None, filter_lines=None, include_files=None
) -> typing.Iterable[Line]:
    """Inspect `git diff` output, yields changed lines."""

    logger = logging.getLogger(__name__)

    if to_ref:
        changeset = "{}...{}".format(from_ref if from_ref else "HEAD", to_ref)
    else:
        changeset = from_ref if from_ref else "HEAD"

    # We're using the pre-commit framework which stashes all unstaged changes
    # before running the pre-commit hooks, so we don't need to add `--cached`
    # here. Also, if we run 2 hooks that modify the files, the second hook
    # should work on the diff that includes the unstaged changes made by the
    # first hook, not the original diff.
    cmd = ["git", "diff", "--patch", "--unified=0", changeset]
    if include_files:
        # If you're committing inside a git worktree, it's possible that files
        # from *outside* the current git repository (i.e. in the original git
        # repository). Therefore we need to filter our all files outside of the
        # current repository before passing them to git diff.
        toplevel_path = get_toplevel_path()
        include_files = {
            path
            for path in include_files
            if os.path.commonprefix((os.path.abspath(path), toplevel_path))
            == toplevel_path
        }
        if not include_files:
            # No files to check
            return

        # If files were moved, it's possible that only the new filename is in
        # the list of included files. For example, this is the case when the
        # script is used by pre-commit. When calling `git diff` after a rename
        # with a path argument where only the new file is listed, git treats
        # the file as newly added and the whole file shows up as added lines.
        #
        # This leads to false positives because the lines were not actually
        # changed. Hence, we check if any of the files were renamed, and make
        # sure that both the old and new filename is included in the initial
        # `git diff` call.
        moved_files = get_moved_files(changeset, include_files=include_files)
        include_files_with_moved = include_files.union(
            itertools.chain.from_iterable(moved_files)
        )
        cmd.extend(["--", *include_files_with_moved])
    logger.debug("Executing: %r", cmd)
    proc = subprocess.run(cmd, capture_output=True)
    proc.check_returncode()
    current_file = None
    hunk_lines_left = 0
    for line in proc.stdout.decode(errors="replace").splitlines():
        match_file = re.match(r"^\+\+\+ b/(.*)$", line)
        if match_file:
            # Current line contains a diff filename
            assert hunk_lines_left == 0
            current_file = match_file.group(1)
            continue

        match_lineno = re.match(
            r"^@@ -(\d+(?:,\d+)?) \+([0-9]+(?:,[0-9]+)?) @@", line
        )
        if match_lineno:
            # Current line contains a hunk header
            assert current_file is not None
            assert hunk_lines_left == 0
            start_removed, _, length_removed = match_lineno.group(1).partition(
                ","
            )
            start_added, _, length_added = match_lineno.group(2).partition(",")
            lineno_removed = int(start_removed)
            lineno_added = int(start_added)
            hunk_lines_left = int(length_removed) if length_removed else 1
            hunk_lines_left += int(length_added) if length_added else 1
            continue

        if hunk_lines_left and line:
            # Current line contains an added/removed line
            hunk_lines_left -= 1

            if line.startswith("+"):
                lineno_added += 1
                current_lineno = lineno_added
            elif line.startswith("-"):
                lineno_removed += 1
                current_lineno = lineno_removed
            else:
                assert line[0] == "\\"
                # This case can happen if the last line is lacking a newline at
                # the EOF, e.g. if the diff looks like this:
                #     -This is the last line.
                #     \ No newline at end of file
                #     +This is the last line.
                continue

            lineobj = Line(
                sourcefile=current_file,
                number=current_lineno - 1,
                text=line[1:],
                added=line.startswith("+"),
            )

            if filter_lines is None or filter_lines(lineobj):
                if include_files and lineobj.sourcefile not in include_files:
                    continue
                yield lineobj

        # If we reach this part, the line does not contain a diff filename or a
        # hunk header and does not belong to a hunk. This means that this line
        # will be ignored implicitly.

    # Make sure we really parsed all lines from the last hunk
    assert hunk_lines_left == 0


def get_changed_lines_grouped(
    from_ref=None, to_ref=None, filter_lines=None, include_files=None
) -> typing.Iterable[FileLines]:
    """Inspect `git diff-index` output, yields lines grouped by file."""

    lines = get_changed_lines(from_ref, to_ref, filter_lines, include_files)
    for filename, file_lines in itertools.groupby(
        lines, key=lambda line: line.sourcefile
    ):
        grouped_linenumbers = []
        start_linenumber = None
        last_linenumber = None
        for line in file_lines:
            if None not in (start_linenumber, last_linenumber):
                if line.number != last_linenumber + 1:
                    grouped_linenumbers.append(
                        (start_linenumber, last_linenumber)
                    )
                    start_linenumber = None

            if start_linenumber is None:
                start_linenumber = line.number
            last_linenumber = line.number

        if None not in (start_linenumber, last_linenumber):
            grouped_linenumbers.append((start_linenumber, last_linenumber))

        if grouped_linenumbers:
            yield FileLines(filename, grouped_linenumbers)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import argparse
import logging
import os
import re
import subprocess
import sys
import tempfile
import typing

import githelper


# We recommend a maximum line length of 80, but do allow up to 100 characters
# if deemed necessary by the developer. Lines that exceed that limit will
# be wrapped after 80 characters automatically.
LINE_LENGTH_THRESHOLD = 100
BREAK_BEFORE = 80


def get_clang_format_config_with_columnlimit(rootdir, limit):
    """Create a temporary config with ColumnLimit set to 80."""
    cpp_file = os.path.join(rootdir, "src/main.cpp")
    proc = subprocess.run(
        ["clang-format", "--dump-config", cpp_file],
        capture_output=True,
        text=True,
    )
    proc.check_returncode()

    return re.sub(
        r"(ColumnLimit:\s*)\d+", r"\g<1>{}".format(limit), proc.stdout
    )


def run_clang_format_on_lines(rootdir, file_to_format, stylepath=None):
    logger = logging.getLogger(__name__)

    line_arguments = [
        "--lines={}:{}".format(start, end)
        for start, end in file_to_format.lines
    ]
    assert line_arguments

    logger.info(
        "Reformatting %s [%s]...",
        file_to_format.filename,
        ", ".join("{}-{}".format(*x) for x in file_to_format.lines),
    )

    filename = os.path.join(rootdir, file_to_format.filename)
    cmd = [
        "clang-format",
        "--style=file",
        # The --assume-filename argument sets the path for the .clang-format
        # config file implicitly by assuming a different location of the file
        # to format
        "--assume-filename={}".format(
            os.path.join(
                stylepath if stylepath else rootdir, file_to_format.filename
            )
        ),
        *line_arguments,
    ]

    with open(filename) as fp:
        logger.debug("Executing: %r", cmd)
        proc = subprocess.run(cmd, stdin=fp, capture_output=True, text=True)
    try:
        proc.check_returncode()
    except subprocess.CalledProcessError:
        logger.error(
            "Error while executing command %s: %s",
            cmd,
            proc.stderr,
        )
        raise

    if proc.stderr:
        logger.error(proc.stderr)
    with open(filename, mode="w") as fp:
        fp.write(proc.stdout)


def main(argv: typing.Optional[typing.List[str]] = None) -> int:
    logging.basicConfig(
        format="[%(levelname)s] %(message)s", level=logging.INFO
    )

    logger = logging.getLogger(__name__)

    parser = argparse.ArgumentParser()
    parser.add_argument("--from-ref", help="use changes changes since commit")
    parser.add_argument("--to-ref", help="use changes until commit")
    parser.add_argument("files", nargs="*", help="only check these files")
    args = parser.parse_args(argv)

    if not args.from_ref:
        args.from_ref = os.getenv("PRE_COMMIT_FROM_REF") or os.getenv(
            "PRE_COMMIT_SOURCE"
        )

    if not args.to_ref:
        args.to_ref = os.getenv("PRE_COMMIT_TO_REF") or os.getenv(
            "PRE_COMMIT_ORIGIN"
        )

    # Filter filenames
    rootdir = githelper.get_toplevel_path()

    # First pass: Format added/changed lines using clang-format
    logger.info("First pass: Reformatting added/changed lines...")
    files_with_added_lines = githelper.get_changed_lines_grouped(
        from_ref=args.from_ref,
        to_ref=args.to_ref,
        filter_lines=lambda line: line.added,
        include_files=args.files,
    )
    for changed_file in files_with_added_lines:
        run_clang_format_on_lines(rootdir, changed_file)

    # Second pass: Wrap long added/changed lines in C++ using clang-format
    logger.info("Second pass: Breaking long added/changed lines...")
    files_with_long_added_lines = githelper.get_changed_lines_grouped(
        from_ref=args.from_ref,
        to_ref=args.to_ref,
        filter_lines=lambda line: line.added
        and len(line.text) > LINE_LENGTH_THRESHOLD,
        include_files=[
            file for file in args.files if re.search(r"\.(?:cpp|h)$", file)
        ],
    )
    config = get_clang_format_config_with_columnlimit(rootdir, BREAK_BEFORE)
    with tempfile.TemporaryDirectory(prefix="clang-format") as tempdir:
        # Create temporary config with ColumnLimit enabled
        configfile = os.path.join(tempdir, ".clang-format")
        with open(configfile, mode="w") as configfp:
            configfp.write(config)

        for changed_file in files_with_long_added_lines:
            run_clang_format_on_lines(rootdir, changed_file, stylepath=tempdir)
    return 0


if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python3

"""
Git Extract Changelog Entries

This script iterates through Git commits and extracts a changelog line for all
merged pull request in the given range to stdout

Usage:
    python script.py --start_commit <start_commit> \
    [--end_commit <end_commit>] [--repo_path /path/to/your/repo]

Dependencies:
    - GitPython library
"""

import git
import re
import argparse


def extract_pull_requests(repo_path, start_commit, end_commit):
    repo = git.Repo(repo_path)
    pr_pattern = re.compile(r"Merge pull request #(\d+)")

    for commit in repo.iter_commits(rev=f"{start_commit}..{end_commit}"):
        match = pr_pattern.search(commit.summary)
        if match:
            pr_number = match.group(1)
            commit_message_lines = commit.message.split("\n")
            print(
                f"* {commit_message_lines[2]} "
                + f"[#{pr_number}]"
                + f"(https://github.com/mixxxdj/mixxx/pull/{pr_number})"
            )


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description="Git Extract Changelog Entries"
    )
    parser.add_argument(
        "--start_commit", required=True, help="Starting commit SHA"
    )
    parser.add_argument(
        "--end_commit", default="HEAD", help="Ending commit SHA (default HEAD)"
    )
    parser.add_argument(
        "--repo_path",
        default=".",
        help="Path to the Git repository (default: current directory)",
    )

    args = parser.parse_args()
    extract_pull_requests(args.repo_path, args.start_commit, args.end_commit)
